******************************************************
** PROJECT     : TORRENT
** TITLE       : TORRENT reviewCCD.
** VERSION     : 7.0
** HISTORY     : VER    DATE     AUTHOR     COMMENTS
**             : 1.0    7/27/10   MH    Taking mark_cdsnoninv.asm and changing to run E2V 
**             : 8/6/10 - removed most stuff not needed - bin, focus, pause
**             : 8/16/10 - added back continuous run
**	       : 8/17/10 - Revised for serial clearing
**	       :	 - revising clocking and cds for real
**             : 11/12/10 - trying to get the clear routine working better
**
** DESCRIPTION : CODE RUNS VIA NORMAL JUMP TABLE SYSTEM 
**             :
**	       : added prescan and dark 9/1/10
**	       :	 modified to remove power supply sync for idle routine to fix HV crash  9/16/10
**  SIGNAL ASSIGNMENTS : for E2V in Tucson - first CCD running for Review!
**  Detector 1
**             : CLK_OUT_00 => D1_P1
**             : CLK_OUT_01 => D1_P2
**             : CLK_OUT_02 => D1_P3
**             : CLK_OUT_03 => D1_S1_L & D1_S1_R
**             : CLK_OUT_04 => D1_S2_L & D1_S2_R
**             : CLK_OUT_05 => D1_S3
**             : CLK_OUT_06 => D1_SW_L & D1_SW_R
**             : CLK_OUT_07 => D1_RG_L & D1_RG_R
**   Detector 2
**             : CLK_OUT_08 => D2_P1
**             : CLK_OUT_09 => D2_P2
**             : CLK_OUT_10 => D2_P3
**             : CLK_OUT_11 => D2_S1_L & D2_S1_R
**             : CLK_OUT_12 => D2_S2_L & D2_S2_R
**             : CLK_OUT_13 => D2_S3
**             : CLK_OUT_14 => D2_SW_L & D2_SW_R
**             : CLK_OUT_15 => D2_RG_L & D2_RG_R
**   end of AFE1 clocks
**   
**     NOTE:	  the E2V CCD44-82 serial register has S1 S2 S3 sequence 
**                 for both registers to read to two outputs!
**
***************************
** CLOCK BIT DEFINITIONS ** 
***************************
DEF #D1_P1           B00000000000000000000000000000001
DEF #D1_P2           B00000000000000000000000000000010
DEF #D1_P3           B00000000000000000000000000000100
DEF #D1_S1           B00000000000000000000000000001000
*
DEF #D1_S2           B00000000000000000000000000010000
DEF #D1_S3           B00000000000000000000000000100000
DEF #D1_SW           B00000000000000000000000001000000
DEF #D1_RG           B00000000000000000000000010000000
*
DEF #D2_P1           B00000000000000000000000100000000
DEF #D2_P2           B00000000000000000000001000000000
DEF #D2_P3           B00000000000000000000010000000000
DEF #D2_S1           B00000000000000000000100000000000
*
DEF #D2_S2           B00000000000000000001000000000000
DEF #D2_S3           B00000000000000000010000000000000
DEF #D2_SW           B00000000000000000100000000000000
DEF #D2_RG           B00000000000000001000000000000000
*
* Combination of D1 and D2 to simplify, since they both are lock stepped
DEF #S1     #D1_S1 OR #D2_S1
DEF #S2     #D1_S2 OR #D2_S2
DEF #S3     #D1_S3 OR #D2_S3
DEF #SW     #D1_SW OR #D2_SW
DEF #RG     #D1_RG OR #D2_RG   
*
** PARALLEL CLOCK BIT PATTERNS - E2V reccomends to integrate under P2
**
DEF #P_STATE_0     #D1_P2 OR #D2_P2                        * HI: P2 - STATIC CONDITION
DEF #P_STATE_1     #D1_P2 OR #D1_P3 OR #D2_P2 OR #D2_P3    * HI: P23
DEF #P_STATE_2     #D1_P3 OR #D2_P3                        * HI: P3
DEF #P_STATE_3     #D1_P1 OR #D1_P3 OR #D2_P1 OR #D2_P3    * HI: P13
DEF #P_STATE_4     #D1_P1 OR #D2_P1                        * HI: P1
DEF #P_STATE_5     #D1_P1 OR #D1_P2 OR #D2_P1 OR #D2_P2    * HI: P12
*
** SERIAL Clock BIT Patterns - E2V reccomends to have S1 & S2  high for the Parallel xfer
**
DEF #S_STATE_0     #S1 OR #S2           * PARALLEL SHIFT & PARALLEL CLEAR: S1 S2
DEF #S_STATE_1     #S2 OR #RG           * START -    S2       RG
DEF #S_STATE_2     #S2 OR #S3 OR #SW    * SER_2 -    S2 S3 SW
DEF #S_STATE_3     #S3 OR #SW           * SER_3 -       S3 SW
DEF #S_STATE_4     #S1 OR #S3 OR #SW    * SER_4 - S1    S3 SW
DEF #S_STATE_5     #S1 OR #SW           * SER_5 - S1    S3 SW
DEF #S_STATE_6     #S1                  * SER_6 - S1 - VIDEO DUMP
*
*
** IDLE SERIAL CLOCKING VECTORS - since we can't use the dump function, 
**  serial dumping is replaced by idle
DEF #IDL_0      #P_STATE_0 OR #S_STATE_2                     
DEF #IDL_1      #P_STATE_0 OR #S_STATE_3
DEF #IDL_2      #P_STATE_0 OR #S_STATE_4
DEF #IDL_3      #P_STATE_0 OR #S_STATE_5
DEF #IDL_4      #P_STATE_0 OR #S_STATE_6
DEF #IDL_5      #P_STATE_0 OR #S_STATE_1
*
** PARALLEL CLOCKING VECTORS 
DEF #PAR_0      #S_STATE_0 OR #P_STATE_0
DEF #PAR_1      #S_STATE_0 OR #P_STATE_1
DEF #PAR_2      #S_STATE_0 OR #P_STATE_2
DEF #PAR_3      #S_STATE_0 OR #P_STATE_3
DEF #PAR_4      #S_STATE_0 OR #P_STATE_4
DEF #PAR_5      #S_STATE_0 OR #P_STATE_5

** SERIAL READOUT VECTORS 
DEF #SER_0      #P_STATE_0 OR #S_STATE_0                     
DEF #SER_1      #P_STATE_0 OR #S_STATE_1
DEF #SER_2      #P_STATE_0 OR #S_STATE_2
DEF #SER_3      #P_STATE_0 OR #S_STATE_3
DEF #SER_4      #P_STATE_0 OR #S_STATE_4
DEF #SER_5      #P_STATE_0 OR #S_STATE_5
DEF #SER_6      #P_STATE_0 OR #S_STATE_6   * Video Dump here
*
** INTEGRATION AND INITIALIZATION CLOCKING VECTORS
DEF #INT_0      #P_STATE_0 OR #S_STATE_0
DEF #INT_1      #P_STATE_0 OR #S_STATE_0
**
**********************
** TIMING CONSTANTS ** 
**********************
DEF #PARALLEL_DELAY 15       * 5 us read parallel phase delay [us]
DEF #PAR_CLEAR_DEL  5       * 5 us reset parallel phase delay [us]
DEF #SERIAL_DELAY   7      * Delay for serial clock [clk], about 250ns
DEF #SER_SKIP_DEL   4       * 4 sequencer clocks = 200ns (clk)???
DEF #INT_TIME       40      * integration time = slope
DEF #PAN_DLY        200     * 500 ms Delay before sending data to PAN
*
**********************
** TIMING CONSTANTS ** from mosaic1
**********************
*DEF #PARALLEL_DELAY 25        * 15 us read parallel phase overlap = 90 us period
DEF #PCLR_DELAY	    40		  * 30 us clearing parallel phase overlap
DEF #SX2PX_DELAY     2		  * Delay from finish of serial xfer to start of parallel transfer (2 us)
DEF #PX2SX_DELAY    20		  * Delay from finish of parallel xfer to start of serial transfer (10 us)
*DEF #INT_TIME        1        * integration time [us] = slope
*DEF #PAN_DLY       200        * 500 ms DELAY BEFORE SENDING DATA TO PAN
DEF #CLEAR_DLY  x00003E8      * Delay will be 100ns x CLEAR_DLY e2 = 100ms
**
*******************************
** LOOP REGISTER DEFINITIONS ** 
*******************************
DEF #NROW          0        * NUMBER OF ROWS TO READOUT (DET_ROWS / NPBIN)
DEF #NCOL          1        * NUMBER OF COLUMNS TO READOUT (DET_COLS / NSBIN)
DEF #ROWSKIP       4        * Number of ROWS to skip for IDLE
DEF #COLSKIP       5        * Number of COLUMNS to skip for IDLE
DEF #NPRE          7        * Number of serial prescan pixels
DEF #SHUTDLY       8        * CLOCK TICKS TO HOLD INT RESET AND DC RESTORE TRUE
DEF #INITDLY       9         * Clock initialization delay
DEF #ROI_ROWS     10        * ROI - number of rows in ROI
DEF #ROI_COLS     11        * ROI - number of columns in ROI
DEF #ROI_PRE      12        * ROI - number of rows to skip up to the ROI
DEF #ROI_SKIP     13        * ROI - number of cols to skip over to the ROI
DEF #SPARE_14     14        * not used
DEF #SPARE_15     15        * not used
**
**
**
**************************************************************
** JUMP FLAG DEFINITIONS - BIT POSITION IN COMMAND REGISTER ** from stis
**************************************************************
DEF #SYNC        1           * Jump on sync bit high
DEF #ITC         2           * Jump on Integration time expired
DEF #START       3           * Jump on start exposure flag set
DEF #CCDREAD     4           * STRVCTR =  1: DO CCD INTEGRATE & READ
DEF #DARKEXP     5           * STRVCTR =  2: Shutter closed during integration then readout
DEF #JREAD       6           * STRVCTR =  4: Just read out
DEF #VCTR_4      7           * STRVCTR =  8: 
DEF #VCTR_5      8           * STRVCTR = 16: 
DEF #VCTR_6      9           * STRVCTR = 32: Spare
DEF #VCTR_7     10           * STRVCTR = 64: Spare
DEF #VCTR_8     11           * STRVCTR = 128: Spare
**   User Bits
DEF #CONT_RUN   12           * Continuous run flag
DEF #USR_2      13           * Serial binning enable bit
DEF #USR_3      14           * Jump on user mode bit value 4
DEF #USR_4      15           * Jump on user mode bit value 8
**
****************************
** EFR REGISTER CONSTANTS ** Used with 16-bit or 32-bit write modes 
****************************
DEF #KILL_EXPFLG    x01080001       * Cancel start exposure flag (bit 3 of Command Register) using the EFR register
DEF #PULSE_PWRSYNC  x01080002       * Pulse the power synchronization strobe for power supply synchronization
DEF #STRT_INTCNTR   x01080004       * Start the integration time counter using the EFR register
DEF #STOP_INTCNTR   x01080008       * Stop the integration time counter using the EFR register
DEF #OPEN_SHUTTER   x01080010       * Command the shutter to open
DEF #CLOSE_SHUTTER  x01080020       * Command the shutter to close
DEF #PREFLASH_ON    x01080040       * Turn the preflash signal on
DEF #PREFLASH_OFF   x01080080       * Turn off the preflash signal
DEF #PULSE_FRAME    x01080100       * Pulse the Frame Start flag to indicate a new frame
DEF #PULSE_LINE     x01080200       * Pulse the Line s Start Flag to indicate a new line beginning
DEF #READOUT_BUSY   x01080400       * Set the readout in progress flag to the DHE - Allows shutting down certain services while reading out.
DEF #READOUT_IDLE   x01080800       * Set the readout in progress flag false
DEF #SPARE_EFR_1    x01081000       * Spare toggle bit - not used in sequencer
DEF #SPARE_EFR_2    x01082000       * Spare toggle bit - not used in sequencer
DEF #SET_SYNC_LOW   x01084000       * If enabled as master DHE, set sync bit low after sync delay
DEF #SET_SYNC_HIGH  x01088000       * If enabled as master DHE, set sync signal high immediately
*
**   COMBINATION EFR COMMAND WORDS             
DEF #OPEN_AND_START      #STRT_INTCNTR OR #OPEN_SHUTTER     * START INTEGRATION TIMER AND OPEN SHUTTER
DEF #CLOSE_AND_STOP      #STOP_INTCNTR OR #CLOSE_SHUTTER    * CLOSE SHUTTER AND STOP INTEGRATION TIMER
DEF #CLOSE_STOP_BUSY     #CLOSE_AND_STOP OR #READOUT_BUSY   * CLOSE SHUTTER, STOP INTEGRATION TIMER, AND SET 'READOUT IN PROGRESS' STATUS
**
*****************************
** MODE REGISTER CONSTANTS **
*****************************
DEF #RESET       0                  * Mode register constant for board reset operation - Not used in sequencer
DEF #READ        1                  * Mode register constant for read operation - Not used in sequencer
DEF #WRITE16     2                  * Mode register constant for 16 bit write operation - Module address contained in upper 16-bits of pattern data
DEF #WRITE32     3                  * Mode register constant for 32 bit write operation - use LDA to adress module

**
*******************************
** LSR BOARD SELECT ASSIGNMENTS ** combined
*******************************
DEF #DSLCT       0
DEF #LCB         1
DEF #PSM         2
DEF #CFG         4
DEF #PIX         8
DEF #AFE        16

*******************************
** CFG MODULE REGISTER ADDRESSING
DEF #EFR_REG     x0108              * WRITE ALL EFR PARAMETERS TO THIS REGISTER WHEN USING 32-BIT MODE
DEF #TSTCOND_REG x0109              * WRITE A 23-BIT ADDRESS HERE TO SEE the RESULT OF A READ OF THAT ADDRESS HERE

*******************************
** TEST CONDITION READ ADDRESSES
DEF #RD_STATUS x0001fffd            * read the system ststus word

*******************************
** AFE Sub addresses
DEF #CLKPORT     x3002  ** AFE MODULE CLOCK ADDRESS POINTERS
DEF #CDSREG      x2002  ** AFE MODULE CDS ADDRESS POINTERS
	
*******************************
** CDS state values - written as data for 16-bit mode writes to the cds register
*******************************
DEF #CDS_CTC x20020001    * CTC strobe
DEF #CDS_INV x20020002    * INVERT mode
DEF #CDS_NON x20020004    * NON_INVERT mode
DEF #CDS_INT x20020008    * Integrate
DEF #CDS_CLP x20020010    * DC Restore Clamp
DEF #CDS_RST x20020020    * RESET Integratoe

DEF #CDS00 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
DEF #CDS01 #CDS_INV OR #CDS_CLP OR #CDS_RST
DEF #CDS02 #CDS_INV OR #CDS_RST
DEF #CDS03 #CDS_INV
DEF #CDS04 #CDS_INV OR #CDS_INT
DEF #CDS05 #CDS_INV
DEF #CDS06 #CDS_INV OR #CDS_NON
DEF #CDS07 #CDS_NON
DEF #CDS08 #CDS_NON OR #CDS_INT
DEF #CDS09 #CDS_NON
DEF #CDS10 #CDS_NON OR #CDS_CTC
DEF #CDS11 #CDS_INV OR #CDS_NON OR #CDS_CLP
**
********************************
** PATTERN MEMORY DEFINITIONS ** combined
********************************
** PATTERN MEMORY SEGMENTATION MAP
DEF #PATSEG_00      x0000         * Initialization stuff
DEF #PATSEG_INIT    x0020         * CCD READ INITIALIZATION
DEF #PATSEG_PAR     x0030         * CCD PARALLEL SHIFT VECTORS
DEF #PATSEG_SER     x0040         * CCD SERIAL SHIFT VECTORS
DEF #PATSEG_CDS     x0080         * CDS vectors
DEF #PATSEG_COND    x00A0         * Test condition read addresses
DEF #PATSEG_ROI     x00C0         * Place to put all ROI shifting

ORG PAT #PATSEG_00                    ********** GENERAL CONSTANTS KEPT IN PAT MEM ********* 
PAT #KILL_EXPFLG     #KILL_EXPFLG     ** We can use the same label name here because they are in different contexts
PAT #PULSE_PWRSYNC   #PULSE_PWRSYNC   ** the definition earlier points to a data value. the label here points to
PAT #STRT_INTCNTR    #STRT_INTCNTR    ** a pattern memory address
PAT #STOP_INTCNTR    #STOP_INTCNTR    ********************
PAT #OPEN_SHUTTER    #OPEN_SHUTTER    
PAT #CLOSE_SHUTTER   #CLOSE_SHUTTER   
PAT #PREFLASH_ON     #PREFLASH_ON     
PAT #PREFLASH_OFF    #PREFLASH_OFF    
PAT #PULSE_FRAME     #PULSE_FRAME     
PAT #PULSE_LINE      #PULSE_LINE      
PAT #READOUT_BUSY    #READOUT_BUSY    
PAT #READOUT_IDLE    #READOUT_IDLE    
PAT #SET_SYNC_LOW    #SET_SYNC_LOW    
PAT #SET_SYNC_HIGH   #SET_SYNC_HIGH  
PAT #OPEN_AND_START  #OPEN_AND_START
PAT #CLOSE_AND_STOP  #CLOSE_AND_STOP
PAT #CLOSE_STOP_BUSY #CLOSE_STOP_BUSY
                  
ORG PAT #PATSEG_INIT      ** CCD INTEGRATION AND READ INITIALIZATION ********* 
PAT #INT_0      #INT_0
PAT #INT_1      #INT_1

ORG PAT #PATSEG_PAR       ** CCD PARALLEL SHIFT VECTORS ********* 
PAT #PAR_0      #PAR_0
PAT #PAR_1      #PAR_1
PAT #PAR_2      #PAR_2
PAT #PAR_3      #PAR_3
PAT #PAR_4      #PAR_4
PAT #PAR_5      #PAR_5

ORG PAT #PATSEG_SER      ** CCD SERIAL SHIFT BOTH READ & IDLE VECTORS ********* 
PAT #SER_0      #SER_0
PAT #SER_1      #SER_1
PAT #SER_2      #SER_2
PAT #SER_3      #SER_3
PAT #SER_4      #SER_4
PAT #SER_5      #SER_5
PAT #SER_6      #SER_6

PAT #IDL_0      #IDL_0                     
PAT #IDL_1      #IDL_1
PAT #IDL_2      #IDL_2
PAT #IDL_3      #IDL_3
PAT #IDL_4      #IDL_4
PAT #IDL_5      #IDL_5
	
ORG PAT #PATSEG_CDS      ** CDS Control vectors ********* 
PAT #CDS00      #CDS00
PAT #CDS01      #CDS01
PAT #CDS02      #CDS02
PAT #CDS03      #CDS03
PAT #CDS04      #CDS04
PAT #CDS05      #CDS05
PAT #CDS06      #CDS06
PAT #CDS07      #CDS07
PAT #CDS08      #CDS08
PAT #CDS09      #CDS09
PAT #CDS10      #CDS10
PAT #CDS11      #CDS11

ORG PAT #PATSEG_COND
PAT #RD_STATUS  #RD_STATUS  ** READ CURRENT SYSTEM STATUS

ORG PAT #PATSEG_ROI          ** A place to start the ROI wqork
PAT #ROI_ROWS    #ROI_ROWS   
PAT #ROI_COLS    #ROI_COLS   
PAT #ROI_PRE     #ROI_PRE    
PAT #ROI_SKIP    #ROI_SKIP   
**
*************************************
**   Program code  starts here     **
*************************************
** EXECUTIVE ROUTINE - get things initialized and going
**  E2V needs a special startup  **
*
#INIT   NOP                     * Give a bit if time to set things up
	 LRB #INITDLY           * This delay holds clocks in a static init state to help clear the E2V
    	DMS 10                  * Pattern is defined in mode file & is done before starting sequencer
	  LPE
       CAL #DARK                * Establish the initial conditions

#EXEC   LMR #WRITE32
        LPP #RD_STATUS
        LDA #TSTCOND_REG        * READ the SYSTEM STATUS - TEST CODE for new feature
        LSR #CFG
	                    
#IDLE   JCB #GOBABY #START      * IF START EXPOSURE FLAG SET
	CAL #CLEAR              * SCAN OUT ONE LINE
	DMS 1
        JMP #IDLE               * ELSE LOOP and keep looking
  
#GOBABY LMR #WRITE16            * SET 16 Bit WRITE mode for EFR WRITES
        LPP #KILL_EXPFLG        * KILL the START EXP FLAG - PATTERN = 0x00000001
        LSR #CFG                * WRITE to the EFR
        JCB #JRDOUT  #CCDREAD   * Do CCD Integrate & readout
        JCB #JRDARK  #DARKEXP   * Do CCD Integrate with shutter closed & readout
        JMP #EXEC               * NO FLAGS SET, back to EXEC loop
                
** READOUT ROUTINES
* START VECTOR = 1 - CCD INTEGRATE and READOUT *****
*
#JRDOUT NOP        
        LMR #WRITE16
        LPP #OPEN_AND_START     * OPEN SHUTTER and START the INTEGRATION TIMER
        LSR #CFG                * Start INTEGRATION Timer running
        NOP                     * Delay for bug that prevents ITC flag change to be seen in one clock cycle
        LMR #WRITE32
#RD1    JCB #RD2 #ITC
        JMP #RD1                * LOOP LOCALLY WHILE WAITING FOR INTEGRATION TIMER TO TIME OUT
#RD2    LMR #WRITE16            * JUST IN CASE WE HAVE BECOME CONFUSED
        LPP #CLOSE_STOP_BUSY    * CLOSE the SHUTTER, STOP the INTEGARTION TIMER, GOTO 'READOUT IN PROGRESS' STATUS
        LSR #CFG                * Stop INTEGRATION Timer
        CAL #READ               * READOUT WHEN ITC HAS REACHED REQUIRED TIME
        JCB #JRDOUT #CONT_RUN   * Do it all again IF CONT_RUN set
        JMP #EXEC               * CALL IT A DAY

#JRDARK NOP        
        LMR #WRITE16
        LPP #STRT_INTCNTR       * OPEN SHUTTER and START the INTEGRATION TIMER
        LSR #CFG                * Start INTEGRATION Timer running
        NOP                     * Delay for bug that prevents ITC flag change to be seen in one clock cycle
        LMR #WRITE32
#RD1    JCB #RD2 #ITC
        JMP #RD1                * LOOP LOCALLY WHILE WAITING FOR INTEGRATION TIMER TO TIME OUT
#RD2    LMR #WRITE16            * JUST IN CASE WE HAVE BECOME CONFUSED
        LPP #CLOSE_STOP_BUSY    * CLOSE the SHUTTER, STOP the INTEGARTION TIMER, GOTO 'READOUT IN PROGRESS' STATUS
        LSR #CFG                * Stop INTEGRATION Timer
        CAL #READ               * READOUT WHEN ITC HAS REACHED REQUIRED TIME
        JCB #JRDOUT #CONT_RUN   * Do it all again IF CONT_RUN set
        JMP #EXEC               * CALL IT A DAY
	
*******************************************************************      
** SUBROUTINES 
*******************************************************************  
*
*
#CLEAR   NOP                    * Just shift a line and a row
            CAL #P_CLEAR
            CAL #SER_CLEAR
         RET

#READ   CAL #RINIT              * INITIALIZE FOR READ
        CAL #SER_CLEAR          * CLEAR the SERIAL (=HORIZONTAL) REGISTER
        LRB #NROW               * LOOP FOR N ROWS
           CAL #PAR_SHFT        * DO ROW XFERS FOR N BINS
    	   CAL #PRESCAN
           LMR #WRITE16
           LPP #PULSE_LINE
           LSR #CFG             * INDICATE A NEW ROW TO the ACQUISITION HARDWARE
           LRB #NCOL            * LOOP FOR N COLS
             LMR #WRITE16
             LPP #PULSE_PWRSYNC       
             LSR #CFG           * SYNCHRONIZE the POWER SUPPLY
             CAL #SER_READ      * DO COLUMN READ FOR N BINS
           LPE                  * END OF COLUMN LOOP
        LPE                     * END OF ROW LOOP
        CAL #INTEG              * PUT BACK INTO INTEGRATE MODE AND CANCEL READOUT ACTIVE STATUS
       RET

#INTEG  LMR #WRITE32            * SET 32 BIT WRITE MODE
        LDA #CLKPORT
        LPP #INT_0              * SET P2
        LSR #AFE                * ESTABLISH the CORRECT CLOCK PHASES FOR INTEGRATION
        LDA #EFR_REG
        LPP #READOUT_IDLE
        LSR #CFG                * KILL ANY EFR FLAGS AND SETUP FOR NEXT EXPOSURE
       RET

#DARK   LMR #WRITE32            * SET 32 BIT WRITE Mode
        LDA #CLKPORT
        LPP #INT_1              * Set Clocks to integrate mode with shutter closed
        LSR #AFE                * Establish the clock phasing for integration
       RET
        

#RINIT  LMR #WRITE32
        LDA #EFR_REG
        LPP #READOUT_BUSY
        LSR #CFG                 * SET READOUT BUSY
        LPP #PULSE_FRAME
        LSR #CFG                 * SET the NEW FRAME FLAG FOR the ACQUISITION FIRMWARE
      RET

#P_CLEAR LMR #WRITE32
         LRB #ROWSKIP            * Do this for ROW Skip number for IDLE - shifts multiple rows in
         LDA #CLKPORT
           LPP #PAR_1   
           LSR #AFE  
         LDA #CLKPORT                        
           DUS #PARALLEL_DELAY   *  one parallel delay time 
           LPP #PAR_2  
           LSR #AFE          
         LDA #CLKPORT              
           DUS #PARALLEL_DELAY   *  one parallel delay time 
           LPP #PAR_3  
           LSR #AFE          
         LDA #CLKPORT              
           DUS #PARALLEL_DELAY   *  one parallel delay time 
           LPP #PAR_4   
           LSR #AFE          
         LDA #CLKPORT             
           DUS #PARALLEL_DELAY   *  one parallel delay time 
           LPP #PAR_5   
           LSR #AFE          
         LDA #CLKPORT               
           DUS #PARALLEL_DELAY   *  one parallel delay time 
           LPP #PAR_0   
           LSR #AFE                          
           DUS #PARALLEL_DELAY   *  one parallel delay time 
        LPE
      RET
       

#PAR_SHFT LMR #WRITE32
*        LRB #NPBIN              * Load parallel binning - not implemented
        LDA #CLKPORT
	   LPP #PAR_1            * 
	   LSR #AFE                
          DUS #PARALLEL_DELAY    *  one parallel delay time
        LDA #CLKPORT 
           LPP #PAR_2            *  
           LSR #AFE                
          DUS #PARALLEL_DELAY    *  one parallel delay time
        LDA #CLKPORT
           LPP #PAR_3            *  
           LSR #AFE                
          DUS #PARALLEL_DELAY    *  one parallel delay time
        LDA #CLKPORT
           LPP #PAR_4            *  
           LSR #AFE                 
          DUS #PARALLEL_DELAY    *  one parallel delay time
        LDA #CLKPORT
           LPP #PAR_5            *  
           LSR #AFE                 
          DUS #PARALLEL_DELAY    *  one parallel delay time
        LDA #CLKPORT
           LPP #PAR_0            *  
           LSR #AFE                 
          DUS #PARALLEL_DELAY    *  one parallel delay time
*        LPE                     *  would be the end of parallel loop 
     RET

#SER_READ NOP                    * read one pixel: shift, process, A/D convert, xmit
	LMR #WRITE32             * Set things up
        LDA #CDSREG              * Comes in with S1 S2 
      LPP #CDS00                 * CDS00 Put the CDS into safe mode 
        LSR #AFE
	
        LDA #CDSREG
      LPP #CDS01                 * CDS01 - inv, clp & rst
        LSR #AFE
        
        LDA #CLKPORT
      LPP #SER_0                 * S1 S2
        LSR #AFE

        DSC #SERIAL_DELAY        *  one serial delay time
        LDA #CLKPORT
      LPP #SER_1                 *  S2 RG -> Reset here
	LSR #AFE 

        LDA #CDSREG
      LPP #CDS02                 * CDS02 - inv & rst
        LSR #AFE
	
*      DSC #SERIAL_DELAY          * one serial delay time 
        LDA #CLKPORT
      LPP #SER_2                 * S2 S3 SW
        LSR #AFE 

      DSC 2
        LDA #CDSREG
      LPP #CDS03                 * CDS03 - inv
        LSR #AFE

      DSC 2	                 * one serial delay time 
        LDA #CLKPORT
      LPP #SER_3                 * S3 SW
        LSR #AFE 

      DSC #SERIAL_DELAY          * one serial delay time 
        LDA #CLKPORT
      LPP #SER_4                 * S1 S3 SW
        LSR #AFE 

      DSC #SERIAL_DELAY          * one serial delay time 
        LDA #CLKPORT
      LPP #SER_5                 * S1 SW
        LSR #AFE

        LDA #CDSREG
      LPP #CDS04                 * CDS04 - integrate on reset - inv & int
        LSR #AFE

      DSC #INT_TIME              * Delay for CDS

        LDA #CDSREG
      LPP #CDS05                 * CDS05 - end integrate on reset - inv 
        LSR #AFE

*        LDA #CDSREG
*      LPP #CDS06                * CDS06 -  inv & noninv
*       LSR #AFE

	DSC 2

        LDA #CDSREG
      LPP #CDS07                 * CDS07 - noninv
        LSR #AFE

	DSC 2
	        
        LDA #CLKPORT
      LPP #SER_6                 * S1 -> Video Dump
        LSR #AFE 
        
      DSC 10                     * Let video settle
        LDA #CDSREG
      LPP #CDS08                 * CDS08 - integrate on signal - noninv & int
        LSR #AFE

      DSC #INT_TIME              * Delay for CDS

        LDA #CDSREG
      LPP #CDS09                 * CDS09 - end integration - nonint
        LSR #AFE

      DSC 25		         * wait for video to settle before CTC
        LDA #CDSREG
      LPP #CDS10                 * CDS10 - Send CTC - noninv & ctc
        LSR #AFE

      DSC 15
        LDA #CDSREG
      LPP #CDS11                 * CDS11 - Normalize the CDS - inv, noninv & clp
        LSR #AFE

         RET

#SER_CLEAR NOP
	CAL #PRESCAN             * get rid of prescan pixels
 	 LRB #NCOL               * Shift NCOL columns to speed things up, normally 4
       LMR #WRITE32              * Set things up
	
        LDA #CDSREG              * Comes in with S1 S2 
      LPP #CDS00                 * CDS00 Put the CDS into safe mode 
        LSR #AFE   

      DSC 2                      * Short delay
        LDA #CDSREG
      LPP #CDS01                 * CDS01 - inv, clp & rst
        LSR #AFE
        
        LDA #CLKPORT
      LPP #SER_0                 * S1 S2
        LSR #AFE

      DSC 5			 *  one serial delay time
        LDA #CLKPORT
      LPP #SER_1                 *  S2 RG -> Reset here
        LSR #AFE 

        LDA #CDSREG
      LPP #CDS02                 * CDS02 - inv & rst
        LSR #AFE
	
      DSC 5			 * one serial delay time 
        LDA #CLKPORT
      LPP #SER_2                 * S2 S3 SW
        LSR #AFE 

      DSC 2
        LDA #CDSREG
      LPP #CDS03                 * CDS03 - inv
        LSR #AFE

      DSC 5			 * one serial delay time 
        LDA #CLKPORT
      LPP #SER_3                 * S3 SW
        LSR #AFE 
	
      DSC 5			 * one serial delay time 
        LDA #CLKPORT
      LPP #SER_4                 * S1 S3 SW
        LSR #AFE 

      DSC 5			 * one serial delay time 
        LDA #CLKPORT
      LPP #SER_5                 * S1 SW
        LSR #AFE

      DSC 5
        LDA #CDSREG
      LPP #CDS04                 * CDS04 - integrate on reset - inv & int
        LSR #AFE

      DSC 5			 * Delay for CDS
        LDA #CDSREG
      LPP #CDS05                 * CDS05 - end integrate on reset - inv 
        LSR #AFE

      DSC 5
        LDA #CDSREG
      LPP #CDS06                 * CDS06 -  inv & noninv
        LSR #AFE

      DSC 5
        LDA #CDSREG
      LPP #CDS07                 * CDS07 - noninv
        LSR #AFE
        
      DSC 5
        LDA #CLKPORT
      LPP #SER_6                 * S1 -> Video Dump
        LSR #AFE 
        
      DSC 5                      * Let video settle
        LDA #CDSREG
      LPP #CDS08                 * CDS08 - integrate on signal - noninv & int
        LSR #AFE

      DSC 5E                     * Delay for CDS
        LDA #CDSREG
      LPP #CDS09                 * CDS09 - end integration - nonint
        LSR #AFE

      DSC 5
        LDA #CDSREG
      LPP #CDS11                 * CDS11 - Normalize the CDS - inv, noninv & clp
        LSR #AFE

      DSC 5     
        LDA #CLKPORT
      LPP #SER_0                 * S1 S2
        LSR #AFE
     LPE
    RET

#PRESCAN NOP
        LMR #WRITE32       * Set things up
	 LRB #NPRE         * Load # of prescan colums, 50 for the E2V

          LDA #CLKPORT
	  LPP #SER_0       * S1 S2
	  LSR #AFE

	DSC 10  
	  LDA #CLKPORT
	  LPP #SER_1       *  S2 RG -> Reset here
	  LSR #AFE 

	DSC 10  
	  LDA #CLKPORT
	  LPP #SER_2       * S2 S3 SW
	  LSR #AFE 

	DSC 10  
	  LDA #CLKPORT
	  LPP #SER_3       * S3 SW
	  LSR #AFE 

	DSC 10  
	  LDA #CLKPORT
	  LPP #SER_4       * S1 S3 SW
	  LSR #AFE 

	DSC 10  
 	  LDA #CLKPORT
	  LPP #SER_5       * S1 SW
	  LSR #AFE

	DSC 10
	  LDA #CLKPORT
	  LPP #SER_6       * S1 -> Video Dump
	  LSR #AFE 
       	LPE
       RET
	
* END OF PROGRAM SEGMENT
