******************************************************
** PROJECT     : TORRENT
** TITLE       : chile1 Design Review sequencer.
** VERSION     : 1.0
** HISTORY     : VER    DATE     AUTHOR     COMMENTS
**             : 1.0    8/05/11   PCM    Adapting kosmos_e2v_Sequencer for design verification work
**             : 
** DESCRIPTION : This code is used to validate the design of the Torrent DHE hardware. It contains various test sequences and a suit of E2V 44-82 readout routines for demo 
**             :
**             : Compile as tasm -achile_cds.ucd chile1_Sequencer.asm chile1_Sequencer.ucd
**             :
**
** SIGNAL ASSIGNMENTS : Modified from kosmos to provide the dump gate on a proper clock rail
**#    Configuration Document: TRNT-EL-05-0203_WireList for System chile1
**#    sysConfig Version 1.03 of Wednesday 20110608:1554
**#
**#    WireList for system chile1
**#
**#    Responsible Engineer: Peter Moore
**#    System User: NOAO Detector Group
**#
**#    Use 24 gauge Teflon wires in standard Torrent colors
**#    Clocks - white,          Clock Rtns - white/black
**#    LV Biases - yellow       LV Bias Rtns - brown
**#    HV biases - orange       HV Bias Rtns - black
**#    Video signals - violet   Video Rtns - white/violet
**#    Aux signals - grey,      Aux Rtns - white/grey
**#    Heater - blue,           Heater Rtn - white/blue
**#    TC+ - blue copper,       TC- - red constantin
**#
**#CNCT Det Sgnl - Array Pins => Dewar Pins =>   Dhe Pins      :Dhe Function Name    
**CNCT     SUB -    XNM1:C1 =>      JP3:j =>      J7:03      :afe1HVBiasRtn0       
**CNCT     SUB -    XNM1:C8 =>      JP3:T =>      J8:09      :afe1LVBiasRtn0       
**CNCT     SUB -    XNM1:F2 =>      JP3:f =>      J6:37      :afe1ClkRtn0          
**CNCT      P1 -    XNM1:D8 =>      JP3:A =>      J6:11      :afe1Clk12            
**CNCT      P2 -    XNM1:E8 =>      JP3:B =>      J6:09      :afe1Clk13            
**CNCT      P3 -    XNM1:F8 =>      JP3:C =>      J6:07      :afe1Clk14            
**CNCT    S1_L -    XNM1:D4 =>      JP3:D =>      J6:33      :afe1Clk01            
**CNCT    S2_L -    XNM1:E4 =>      JP3:E =>      J6:31      :afe1Clk02            
**CNCT    S1_R -    XNM1:D5 =>      JP3:F =>      J6:25      :afe1Clk05            
**CNCT    S2_R -    XNM1:E5 =>      JP3:G =>      J6:23      :afe1Clk06            
**CNCT      S3 -    XNM1:F6 =>      JP3:U =>      J6:19      :afe1Clk08            
**CNCT    SW_L -    XNM1:E2 =>      JP3:V =>      J6:29      :afe1Clk03            
**CNCT    SW_R -    XNM1:E7 =>      JP3:X =>      J6:21      :afe1Clk07            
**CNCT    RG_L -    XNM1:E3 =>      JP3:W =>      J6:35      :afe1Clk00            
**CNCT    RG_R -    XNM1:E6 =>      JP3:Y =>      J6:27      :afe1Clk04            
**CNCT      DG -    XNM1:F3 =>      JP3:g =>      J6:05      :afe1Clk15            
**CNCT   OG1_L -    XNM1:D3 =>      JP3:N =>      J8:01      :afe1LVBias00         
**CNCT   OG1_R -    XNM1:D6 =>      JP3:c =>      J8:13      :afe1LVBias04         
**CNCT   OG2_L -    XNM1:D2 =>      JP3:R =>      J8:03      :afe1LVBias01         
**CNCT   OG2_R -    XNM1:D7 =>      JP3:S =>      J8:15      :afe1LVBias05         
**CNCT    DD_L -    XNM1:B2 =>      JP3:h =>      J7:19      :afe1HVBias06         
**CNCT    DD_R -    XNM1:B7 =>      JP3:e =>      J7:07      :afe1HVBias02         
**CNCT    OD_L -    XNM1:B1 =>      JP3:M =>      J7:01      :afe1HVBias00         
**CNCT    OD_R -    XNM1:B8 =>      JP3:P =>      J7:13      :afe1HVBias04         
**CNCT    RD_L -    XNM1:C2 =>      JP3:b =>      J7:04      :afe1HVBias01         
**CNCT    RD_R -    XNM1:C7 =>      JP3:d =>      J7:16      :afe1HVBias05         
**CNCT  TS1_i+ -   dfCnct:1 =>      JP1:A =>  Util-E:E7      :I_SRC1               
**CNCT  TS1_v+ -   dfCnct:2 =>      JP1:B =>  Util-E:E4      :TEMPSNS1+            
**CNCT  TS1_i- -   dfCnct:3 =>      JP1:C =>  Util-E:E3      :TEMPSNS1-            
**CNCT  TS1_v- -   dfCnct:4 =>      JP1:D =>  Util-E:E5      :I_SRC1_RET           
**CNCT  TS2_i+ -   dfCnct:6 =>      JP2:G => Util-E:E11      :I_SRC2               
**CNCT  TS2_v+ -   dfCnct:7 =>      JP2:G =>  Util-E:E2      :TEMPSNS2+            
**CNCT  TS2_i- -   dfCnct:8 =>      JP2:H =>  Util-E:E1      :TEMPSNS2-            
**CNCT  TS2_v- -   dfCnct:9 =>      JP2:H =>  Util-E:E6      :I_SRC2_RET           
**CNCT  vHtr1+ -  dfCnct:11 =>      JP1:E => Util-E:E12      :VHTR_P               
**CNCT  vHtr1Rtn -  dfCnct:12 =>      JP1:G => Util-E:E14      :VHTR_N               
**
**             : CLK_OUT_00 => RG_L
**             : CLK_OUT_01 => S1_L
**             : CLK_OUT_02 => S2_L
**             : CLK_OUT_03 => SW_L
**             : CLK_OUT_04 => RG_R
**             : CLK_OUT_05 => S1_R
**             : CLK_OUT_06 => S2_R
**             : CLK_OUT_07 => SW_R
**             : CLK_OUT_08 => S3
**             : CLK_OUT_09 => NOT USED
**             : CLK_OUT_10 => NOT USED
**             : CLK_OUT_11 => NOT USED
**             : CLK_OUT_12 => P1
**             : CLK_OUT_13 => P2
**             : CLK_OUT_14 => P3
**             : CLK_OUT_15 => DG
**   
**     NOTE:	  the E2V CCD44-82 serial register has S1 S2 S3 sequence 
**               for both registers to read to two outputs!
**
**
***************************
** CLOCK BIT DEFINITIONS ** 
***************************
DEF #RG_L  B00000000000000000000000000000001
DEF #S1_L  B00000000000000000000000000000010
DEF #S2_L  B00000000000000000000000000000100
DEF #SW_L  B00000000000000000000000000001000
*
DEF #RG_R  B00000000000000000000000000010000
DEF #S1_R  B00000000000000000000000000100000
DEF #S2_R  B00000000000000000000000001000000
DEF #SW_R  B00000000000000000000000010000000
*
DEF #S3_LR B00000000000000000000000100000000
DEF #SPR_1 B00000000000000000000001000000000
DEF #SPR_2 B00000000000000000000010000000000
DEF #SPR_3 B00000000000000000000100000000000
*
DEF #P1    B00000000000000000001000000000000
DEF #P2    B00000000000000000010000000000000
DEF #P3    B00000000000000000100000000000000
DEF #DG    B00000000000000001000000000000000
*
* Combination of left and right serial clocks to simplify pattern generation, since they both are lock stepped
DEF #S1            #S1_L OR #S1_R
DEF #S2            #S2_L OR #S2_R
DEF #S3            #S3_LR
DEF #SW            #SW_L OR #SW_R
DEF #RG            #RG_L OR #RG_R   
*
** PARALLEL CLOCK BIT PATTERNS - E2V reccomends to integrate under P2
**
DEF #P_STATE_0     #P2                 * HI: P2 - STATIC CONDITION
DEF #P_STATE_1     #P2 OR #P3          * HI: P23
DEF #P_STATE_2     #P3                 * HI: P3
DEF #P_STATE_3     #P1 OR #P3          * HI: P13
DEF #P_STATE_4     #P1                 * HI: P1
DEF #P_STATE_5     #P1 OR #P2          * HI: P12
*
** SERIAL Clock BIT Patterns - E2V recommends to have S1 & S2  high for the Parallel xfer
**
DEF #S_STATE_0     #S1 OR #S2 OR #SW   * S1 S2    SW
DEF #S_STATE_1     #S2 OR #RG OR #SW   *    S2    SW RG
DEF #S_STATE_2     #S2 OR #S3 OR #SW   *    S2 S3 SW
DEF #S_STATE_3     #S3 OR #SW          *       S3 SW
DEF #S_STATE_4     #S1 OR #S3 OR #SW   * S1    S3 SW
DEF #S_STATE_5     #S1 OR #SW          * S1       SW
DEF #S_STATE_6     #S1                 * S1               - VIDEO DUMP
*
** IDLE SERIAL CLOCKING VECTORS - since we can't use the dump function, 
*
** PARALLEL CLOCKING VECTORS 
DEF #PAR_0         #S_STATE_0 OR #P_STATE_0 OR #RG 
DEF #PAR_1         #S_STATE_0 OR #P_STATE_1 OR #RG
DEF #PAR_2         #S_STATE_0 OR #P_STATE_2 OR #RG
DEF #PAR_3         #S_STATE_0 OR #P_STATE_3 OR #RG
DEF #PAR_4         #S_STATE_0 OR #P_STATE_4 OR #RG
DEF #PAR_5         #S_STATE_0 OR #P_STATE_5 OR #RG
*
** PARALLEL CLEAR CLOCKING VECTORS 
DEF #PAR_CLR0     #S_STATE_0 OR #P_STATE_0 OR #RG OR #DG
DEF #PAR_CLR1     #S_STATE_0 OR #P_STATE_1 OR #RG OR #DG
DEF #PAR_CLR2     #S_STATE_0 OR #P_STATE_2 OR #RG OR #DG
DEF #PAR_CLR3     #S_STATE_0 OR #P_STATE_3 OR #RG OR #DG
DEF #PAR_CLR4     #S_STATE_0 OR #P_STATE_4 OR #RG OR #DG
DEF #PAR_CLR5     #S_STATE_0 OR #P_STATE_5 OR #RG OR #DG
*
** SERIAL READOUT VECTORS 
DEF #SER_0         #P_STATE_0 OR #S_STATE_0                     
DEF #SER_1         #P_STATE_0 OR #S_STATE_1
DEF #SER_2         #P_STATE_0 OR #S_STATE_2
DEF #SER_3         #P_STATE_0 OR #S_STATE_3
DEF #SER_4         #P_STATE_0 OR #S_STATE_4
DEF #SER_5         #P_STATE_0 OR #S_STATE_5
DEF #SER_6         #P_STATE_0 OR #S_STATE_6

* SERIAL REGISTER CLEAR CLOCK VECTORS
DEF #SER_CLR0      #P_STATE_0 OR #S_STATE_0 OR #RG                   
DEF #SER_CLR1      #P_STATE_0 OR #S_STATE_1 OR #RG
DEF #SER_CLR2      #P_STATE_0 OR #S_STATE_2 OR #RG
DEF #SER_CLR3      #P_STATE_0 OR #S_STATE_3 OR #RG
DEF #SER_CLR4      #P_STATE_0 OR #S_STATE_4 OR #RG
DEF #SER_CLR5      #P_STATE_0 OR #S_STATE_5 OR #RG
DEF #SER_CLR6      #P_STATE_0 OR #S_STATE_6 OR #RG

*
** INTEGRATION AND INITIALIZATION CLOCKING VECTORS
*
DEF #INTEG_STATE  #P_STATE_0 OR #S_STATE_0
*
**********************
** TIMING CONSTANTS ** 
**********************
DEF #PARALLEL_DLY    17      * 17 us read parallel phase delay [us] to provide 51us total parallel pulse width (data sheet twi spec) [us]
DEF #TDIR_DLY         3       * difference between 20us and PARALLEL_DLY to comply to data sheet tdir spec [us]
DEF #TDRI_DLY         2       * comply with spec sheet tdri delay period [us]
DEF #PARALLEL_CLR_DLY 7       * 7 us reset parallel phase delay [us]
DEF #SERIAL_DLY       4       * Delay for serial clock, approx. 150ns [seq clk]
DEF #SER_CLR_DLY      2       * Serial clock overlap during serial clear = approx. 220ns (total delay / 37ns - 2) [seq clk]
* DEF #CDS_DWELL      5       * CDS integration time = 300ns [seq clk]
* DEF #CDS_DWELL     13       * CDS integration time = 700ns [seq clk]
* DEF #CDS_DWELL     24       * CDS integration time = 1us [seq clk]
DEF #CDS_DWELL       50       * CDS integration time = 2us [seq clk]
* DEF #CDS_DWELL    103       * CDS integration time = 4us [seq clk]
DEF #ADC_SETTLE      10       * Delay for the signal to settle after last integration period and before ADC samples the result = 520ns [seq clk]
DEF #ADC_HOLD         2       * Hold time for ADC acquisition before clamping down on the video processor approx. 100ns [seq clk]
* DEF #TST_DLY          6
*
*******************************
** LOOP REGISTER DEFINITIONS ** 
*******************************
DEF #NROW             0       * NUMBER OF ROWS TO READOUT (DET_ROWS / NPBIN)
DEF #NCOL             1       * NUMBER OF COLUMNS TO READOUT (DET_COLS / NSBIN)
DEF #NPBIN            2       * Number to bin Parallels
DEF #NSBIN            3       * Number to bin Serials
DEF #ROWSKIP          4       * Number of ROWS to skip for IDLE
DEF #COLSKIP          5       * Number of COLUMNS to skip for IDLE
DEF #NPRE             7       * Number of serial prescan pixels
DEF #SHUTDLY          8       * CLOCK TICKS TO HOLD INT RESET AND DC RESTORE TRUE
DEF #INITDLY          9       * Clock initialization delay
DEF #ROI_ROWS        10       * ROI - number of rows in ROI
DEF #ROI_COLS        11       * ROI - number of columns in ROI
DEF #ROI_PRE         12       * ROI - number of rows to skip up to the ROI
DEF #ROI_SKIP        13       * ROI - number of cols to skip over to the ROI
DEF #TST_DLY         10
**
**
**
**************************************************************
** JUMP FLAG DEFINITIONS - BIT POSITION IN COMMAND REGISTER ** from stis
**************************************************************
DEF #SYNC             1       * Jump on sync bit high
DEF #ITC              2       * Jump on Integration time expired
DEF #START            3       * Jump on start exposure flag set
DEF #CCDREAD          4       * STRVCTR =  1: DO CCD INTEGRATE & READ
DEF #DARKEXP          5       * STRVCTR =  2: Shutter closed during integration then readout
DEF #VCTR_4           6       * STRVCTR =  4: Just read out
DEF #VCTR_8           7       * STRVCTR =  8: 
DEF #VCTR_16          8       * STRVCTR = 16: 
DEF #VCTR_32          9       * STRVCTR = 32: Spare
DEF #VCTR_64         10       * STRVCTR = 64: Spare
DEF #VCTR_128        11       * STRVCTR = 128: Spare
**   User Bits
DEF #CONT_RUN        12       * Continuous run flag
DEF #SBIN_ENB        13       * Serial binning enable bit
DEF #USR_3           14       * Jump on user mode bit value 4
DEF #USR_4           15       * Jump on user mode bit value 8
**
****************************
** EFR REGISTER CONSTANTS ** Used with 16-bit or 32-bit write modes 
****************************
DEF #KILL_EXPFLG    x01080001 * Cancel start exposure flag (bit 3 of Command Register) using the EFR register
DEF #PULSE_PWRSYNC  x01080002 * Pulse the power synchronization strobe for power supply synchronization
DEF #STRT_INTCNTR   x01080004 * Start the integration time counter using the EFR register
DEF #STOP_INTCNTR   x01080008 * Stop the integration time counter using the EFR register
DEF #OPEN_SHUTTER   x01080010 * Command the shutter to open
DEF #CLOSE_SHUTTER  x01080020 * Command the shutter to close
DEF #PREFLASH_ON    x01080040 * Turn the preflash signal on
DEF #PREFLASH_OFF   x01080080 * Turn off the preflash signal
DEF #PULSE_FRAME    x01080100 * Pulse the Frame Start flag to indicate a new frame
DEF #PULSE_LINE     x01080200 * Pulse the Line s Start Flag to indicate a new line beginning
DEF #READOUT_BUSY   x01080400 * Set the readout in progress flag to the DHE - Allows shutting down certain services while reading out.
DEF #READOUT_IDLE   x01080800 * Set the readout in progress flag false
DEF #SPARE_EFR_1    x01081000 * Spare toggle bit - not used in sequencer
DEF #SPARE_EFR_2    x01082000 * Spare toggle bit - not used in sequencer
DEF #SET_SYNC_LOW   x01084000 * If enabled as master DHE, set sync bit low after sync delay
DEF #SET_SYNC_HIGH  x01088000 * If enabled as master DHE, set sync signal high immediately
*
**   COMBINATION EFR COMMAND WORDS             
DEF #OPEN_AND_START      #STRT_INTCNTR OR #OPEN_SHUTTER     * START INTEGRATION TIMER AND OPEN SHUTTER
DEF #CLOSE_AND_STOP      #STOP_INTCNTR OR #CLOSE_SHUTTER    * CLOSE SHUTTER AND STOP INTEGRATION TIMER
DEF #CLOSE_STOP_BUSY     #CLOSE_AND_STOP OR #READOUT_BUSY   * CLOSE SHUTTER, STOP INTEGRATION TIMER, AND SET 'READOUT IN PROGRESS' STATUS
DEF #CLOSE_AND_START     #CLOSE_SHUTTER OR #STRT_INTCNTR    * CLOSE SHUTTER AND START INTEGRATION COUNTER - FOR DARK EXPOSURES
**
*****************************
** MODE REGISTER CONSTANTS **
*****************************
DEF #RESET            0       * Mode register constant for board reset operation - Not used in sequencer
DEF #READ             1       * Mode register constant for read operation - Not used in sequencer
DEF #WRITE16          2       * Mode register constant for 16 bit write operation - Module address contained in upper 16-bits of pattern data
DEF #WRITE32          3       * Mode register constant for 32 bit write operation - use LDA to adress module
**
*******************************
** LSR BOARD SELECT ASSIGNMENTS ** combined
*******************************
DEF #DSLCT            0
DEF #LCB              1
DEF #PSM              2
DEF #CFG              4
DEF #PIX              8
DEF #AFE             16
**
*******************************
** CFG MODULE REGISTER ADDRESSING
DEF #EFR_REG      x0108       * WRITE ALL EFR PARAMETERS TO THIS REGISTER WHEN USING 32-BIT MODE
DEF #TSTCOND_REG  x0109       * WRITE A 23-BIT ADDRESS HERE TO SEE the RESULT OF A READ OF THAT ADDRESS HERE
**
*******************************
** TEST CONDITION READ ADDRESSES
DEF #RD_STATUS x0001fffd            * read the system ststus word
**
*******************************
** AFE Sub addresses
DEF #CLKPORT      x3002       * AFE MODULE CLOCK PORT POINTER
DEF #CDSREG       x2002       * AFE MODULE CDS PORT POINTER
DEF #SEQCMD       x2003       * AFE CDS SEQUENCER TRIGGER POINTER
**
*******************************
** CDS state values - written as data for 16-bit mode writes to the cds register
*******************************
DEF #CDS_CTC  x20020001       * CTC strobe
DEF #CDS_INV  x20020002       * INVERT mode
DEF #CDS_NON  x20020004       * NON_INVERT mode
DEF #CDS_INT  x20020008       * Integrate
DEF #CDS_CLP  x20020010       * DC Restore Clamp
DEF #CDS_RST  x20020020       * RESET Integratoe

*************************
** CDS state sequences **
*************************
*
**Normal readout mode sequence
DEF #CDS00 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
DEF #CDS00 #CDS_INV OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
DEF #CDS01 #CDS_INV OR #CDS_CLP OR #CDS_RST
DEF #CDS02 #CDS_INV OR #CDS_RST
DEF #CDS03 #CDS_INV
DEF #CDS04 #CDS_INV OR #CDS_INT
DEF #CDS05 #CDS_INV
DEF #CDS06 #CDS_INV OR #CDS_NON
DEF #CDS07 #CDS_NON
DEF #CDS08 #CDS_NON OR #CDS_INT
DEF #CDS09 #CDS_NON
DEF #CDS10 #CDS_NON OR #CDS_CTC
DEF #CDS11 #CDS_NON OR #CDS_INT OR #CDS_CLP OR #CDS_RST
DEF #CDS11 #CDS_INV OR #CDS_NON OR #CDS_CLP

**single phase readout mode sequence - just playing
* DEF #CDS00 #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS01 #CDS_NON OR #CDS_CLP
* DEF #CDS02 #CDS_NON OR #CDS_INT
* DEF #CDS03 #CDS_NON
* DEF #CDS04 #CDS_NON OR #CDS_INT
* DEF #CDS05 #CDS_NON
* DEF #CDS06 #CDS_NON OR #CDS_INT
* DEF #CDS07 #CDS_NON
* DEF #CDS08 #CDS_NON OR #CDS_INT
* DEF #CDS09 #CDS_NON
* DEF #CDS10 #CDS_NON OR #CDS_CTC
* DEF #CDS11 #CDS_NON OR #CDS_CLP
* 
* * 
** "Buffer" - DC Restore + Common mode + Integrator held in reset readout mode sequence (i.e. test buffer and ADC)
* DEF #CDS00 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS01 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS02 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS03 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS04 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS05 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS06 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS07 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS08 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS09 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS10 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  OR #CDS_CTC * Normal COMMON Mode rest state + CTC strobe
* DEF #CDS11 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state

*
** "Integrator" - DC Restore + Common Mode readout mode sequence (i.e. Integrator functions on offset signal from inverter stage + Buffer + ADC)
* DEF #CDS00 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS01 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST
* DEF #CDS02 #CDS_INV OR #CDS_NON OR #CDS_RST
* DEF #CDS03 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS04 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_INT
* DEF #CDS05 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS06 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS07 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS08 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_INT
* DEF #CDS09 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS10 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_CTC
* DEF #CDS11 #CDS_INV OR #CDS_NON OR #CDS_CLP

*
** "Inverter" - DC Restore readout mode sequence (i.e. Phase inverter + Integrator functions + buffer + ADC tested)
* DEF #CDS00 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS01 #CDS_INV OR #CDS_CLP OR #CDS_RST
* DEF #CDS02 #CDS_INV OR #CDS_CLP OR #CDS_RST
* DEF #CDS03 #CDS_INV OR #CDS_CLP
* DEF #CDS04 #CDS_INV OR #CDS_CLP OR #CDS_INT
* DEF #CDS05 #CDS_INV OR #CDS_CLP
* DEF #CDS06 #CDS_INV OR #CDS_CLP OR #CDS_NON
* DEF #CDS07 #CDS_NON OR #CDS_CLP
* DEF #CDS08 #CDS_NON OR #CDS_CLP OR #CDS_INT
* DEF #CDS09 #CDS_NON OR #CDS_CLP
* DEF #CDS10 #CDS_NON OR #CDS_CLP OR #CDS_CTC
* DEF #CDS11 #CDS_INV OR #CDS_NON OR #CDS_CLP

** AFE CDS SEQUENCER COMMAND TRIGGER VECTORS
DEF #CDS_NORMAL_READ    x010A00
DEF #CDS_CLMP_READ      
DEF #CDS_CLMP_RST_READ  
**
********************************
** PATTERN MEMORY DEFINITIONS ** combined
********************************
** PATTERN MEMORY SEGMENTATION MAP
DEF #PATSEG_00    x0000       * Initialization stuff
DEF #PATSEG_INIT  x0020       * CCD READ INITIALIZATION
DEF #PATSEG_PAR   x0030       * CCD PARALLEL SHIFT VECTORS
DEF #PATSEG_SER   x0040       * CCD SERIAL SHIFT VECTORS
DEF #PATSEG_CDS   x0080       * CDS vectors
DEF #PATSEG_COND  x00A0       * Test condition read addresses
DEF #PATSEG_ROI   x00C0       * Place to put all ROI shifting

ORG PAT #PATSEG_00                    ********** GENERAL CONSTANTS KEPT IN PAT MEM ********* 
PAT #KILL_EXPFLG     #KILL_EXPFLG     ** We can use the same label name here because they are in different contexts
PAT #PULSE_PWRSYNC   #PULSE_PWRSYNC   ** the definition earlier points to a data value. the label here points to
PAT #STRT_INTCNTR    #STRT_INTCNTR    ** a pattern memory address
PAT #STOP_INTCNTR    #STOP_INTCNTR    ********************
PAT #OPEN_SHUTTER    #OPEN_SHUTTER    
PAT #CLOSE_SHUTTER   #CLOSE_SHUTTER   
PAT #PREFLASH_ON     #PREFLASH_ON     
PAT #PREFLASH_OFF    #PREFLASH_OFF    
PAT #PULSE_FRAME     #PULSE_FRAME     
PAT #PULSE_LINE      #PULSE_LINE      
PAT #READOUT_BUSY    #READOUT_BUSY    
PAT #READOUT_IDLE    #READOUT_IDLE    
PAT #SET_SYNC_LOW    #SET_SYNC_LOW    
PAT #SET_SYNC_HIGH   #SET_SYNC_HIGH  
PAT #OPEN_AND_START  #OPEN_AND_START
PAT #CLOSE_AND_STOP  #CLOSE_AND_STOP
PAT #CLOSE_STOP_BUSY #CLOSE_STOP_BUSY
PAT #CLOSE_AND_START #CLOSE_AND_START
                  
ORG PAT #PATSEG_INIT      ** CCD INTEGRATION AND READ INITIALIZATION ********* 
PAT #INTEG_STATE     #INTEG_STATE

ORG PAT #PATSEG_PAR       ** CCD PARALLEL SHIFT VECTORS ********* 
PAT #PAR_0      #PAR_0
PAT #PAR_1      #PAR_1
PAT #PAR_2      #PAR_2
PAT #PAR_3      #PAR_3
PAT #PAR_4      #PAR_4
PAT #PAR_5      #PAR_5

PAT #PAR_CLR0   #PAR_CLR0
PAT #PAR_CLR1   #PAR_CLR1
PAT #PAR_CLR2   #PAR_CLR2
PAT #PAR_CLR3   #PAR_CLR3
PAT #PAR_CLR4   #PAR_CLR4
PAT #PAR_CLR5   #PAR_CLR5

ORG PAT #PATSEG_SER      ** CCD SERIAL SHIFT BOTH READ & IDLE VECTORS ********* 
PAT #SER_0      #SER_0
PAT #SER_1      #SER_1
PAT #SER_2      #SER_2
PAT #SER_3      #SER_3
PAT #SER_4      #SER_4
PAT #SER_5      #SER_5
PAT #SER_6      #SER_6

PAT #SER_CLR0   #SER_CLR0 
PAT #SER_CLR1   #SER_CLR1
PAT #SER_CLR2   #SER_CLR2
PAT #SER_CLR3   #SER_CLR3
PAT #SER_CLR4   #SER_CLR4
PAT #SER_CLR5   #SER_CLR5
PAT #SER_CLR6   #SER_CLR6
	
ORG PAT #PATSEG_CDS      ** CDS Control vectors ********* 
PAT #CDSCMD     #CDS_NORMAL_READ
PAT #CDS00      #CDS00
PAT #CDS01      #CDS01
PAT #CDS02      #CDS02
PAT #CDS03      #CDS03
PAT #CDS04      #CDS04
PAT #CDS05      #CDS05
PAT #CDS06      #CDS06
PAT #CDS07      #CDS07
PAT #CDS08      #CDS08
PAT #CDS09      #CDS09
PAT #CDS10      #CDS10
PAT #CDS11      #CDS11

ORG PAT #PATSEG_COND
PAT #RD_STATUS  #RD_STATUS  ** READ CURRENT SYSTEM STATUS

ORG PAT #PATSEG_ROI          ** A place to start the ROI wqork
PAT #ROI_ROWS    #ROI_ROWS   
PAT #ROI_COLS    #ROI_COLS   
PAT #ROI_PRE     #ROI_PRE    
PAT #ROI_SKIP    #ROI_SKIP   
*
**
*******************************
** Program code  starts here **
*******************************
** EXECUTIVE ROUTINE - WAIT HERE FOR WORK
*
#INIT   NOP                     * Give the sequencer time to fill teh instruction pipeline
        CAL #INTEG              * Establish the initial conditions

#EXEC   JCB #GOBABY #START      * IF START EXPOSURE FLAG SET GO AND IDENTIFY THE COMMAND 
        CAL #CLR_ROW            * ELSE CLEAR ONE ROW OF THE DETECTOR
        JMP #EXEC               * AND KEEP LOOKING FOR WORK
  
#GOBABY LMR #WRITE32
        LDA #EFR_REG            * POINT TO THE FLAG REGISTER
        LPP #KILL_EXPFLG        * POINT TO THE APPROPRIATE PATTERN DATA TO CANCEL THE START EXP FLAG - PATTERN = 0x00000001
        LSR #CFG                * WRITE FLAG REGISTER IN CONFIG FIRMWARE MODULE
        JCB #JRDOUT   #CCDREAD  * DO NORMAL INTEGRATE AND READOUT - START VECTOR = 1
        JCB #JRDARK   #DARKEXP  * DO DARK INTEGRATE AND READOUT - START VECTOR = 2
        JCB #JVCTR4   #VCTR_4
        JCB #JVCTR8   #VCTR_8
        JCB #JVCTR16  #VCTR_16
        JCB #JVCTR32  #VCTR_32
        JCB #JVCTR64  #VCTR_64
        JCB #JVCTR128 #VCTR_128         
        JMP #EXEC               * NO FLAGS SET, BACK TO THE EXEC LOOP
*
**
**********************         
** READOUT ROUTINES **
**********************
**
*
* START VECTOR = 1 - CLEAR DETECTOR, NORMAL CCD INTEGRATION (SHUTTER CONTROL) AND NORMAL READOUT
#JRDOUT CAL #CLEAR
        CAL #INTEG              * put detector into integration state
        LMR #WRITE32
        LDA #EFR_REG            * point to the flag register
        LPP #OPEN_AND_START     * point to the open shutter and start the integration timer flag register command
        LSR #CFG                * open the shutter and start the integration timer
        NOP                     * delay for bug that prevents itc flag change to be seen in one clock cycle
#RD1    JCB #RD2 #ITC
        JMP #RD1                * loop locally while waiting for integration timer to time out
#RD2    LPP #CLOSE_STOP_BUSY
        LSR #CFG                * close the shutter, stop the integartion timer, and goto 'readout in progress' status
        CAL #READ               * go do the detector readout
        JMP #EXEC               * call it a day
**
* START VECTOR = 2 - CLEAR THE DETector, dark integration (no shutter) and normal readout
#JRDARK  
        CAL #CLEAR              * do a dark integration
        CAL #INTEG              * put detector into integration state
        LPP #CLOSE_AND_START    * close the shutter, start the integration timer
        LSR #CFG
        JMP #RD1

* START VECTOR = 4 - SIMPLE READOUT
#JVCTR4
        CAL #READ
        JCB #JVCTR4 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 8 - 
#JVCTR8
*        JCB #JVCTR8 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC
          
* START VECTOR = 16 - 
#JVCTR16
*        JCB #JVCTR16 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 32 - 
#JVCTR32
*        JCB #JVCTR32 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 64 - 
#JVCTR64
*        JCB #JVCTR64 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 128 - 
#JVCTR128
*        JCB #JVCTR128 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC
*
**        
*****************   
** SUBROUTINES **
*****************  
**
*
#INTEG  LMR #WRITE32  
        LDA #CLKPORT
        LPP #INTEG_STATE
        LSR #AFE                * establish the correct clock phases for integration mode
        LDA #EFR_REG            * point to the flag register
        LPP #READOUT_IDLE       * point to the readout idle flag register command
        LSR #CFG                * set the status (this allows the Dewar heater to come back on line for instance).        
        RET

#RINIT  LMR #WRITE16
        LPP #READOUT_BUSY
        LSR #CFG                 * set readout busy
        LPP #PULSE_FRAME
        LSR #CFG                 * set the new frame flag for the acquisition firmware
        RET

#CLR_ROW 
        CAL #PAR_CLEAR
        RET

#CLEAR  LRB #NROW               * loop for n rows
        CAL #PAR_CLEAR         * do row xfers for n bins
        LPE
        RET
        
#READ   CAL #RINIT              * initialize for read
        CAL #SER_CLEAR          * clear the serial (=horizontal) register
        LRB #NROW               * loop for n rows
        CAL #PAR_SHFT           * do row xfers for n bins
        LMR #WRITE16
        LPP #PULSE_LINE
        LSR #CFG                * indicate a new row to the acquisition hardware
        LRB #NCOL               * loop for n cols
        LMR #WRITE16
        LPP #PULSE_PWRSYNC       
        LSR #CFG                * synchronize the power supply
*        CAL #CDS_TEST
        CAL #SER_READ           * do column read for n bins
        LPE                     * end of column loop
        LPE                     * end of row loop
        CAL #INTEG              * put back into integrate mode and cancel readout active status
        RET

#PAR_SHFT 
        LMR #WRITE32
        LDA #CLKPORT
        LPP #SER_CLR0            * Put serials into proper state (S1 + S2 + RG) before shift occurs
        LSR #AFE
        DUS #TDRI_DLY
        LRB #NPBIN               * Loop for parallel binning value
	     LPP #PAR_1          
	     LSR #AFE            
        DUS #PARALLEL_DLY   
        LPP #PAR_2          
        LSR #AFE            
        DUS #PARALLEL_DLY   
        LPP #PAR_3          
        LSR #AFE            
        DUS #PARALLEL_DLY   
        LPP #PAR_4          
        LSR #AFE            
        DUS #PARALLEL_DLY   
        LPP #PAR_5          
        LSR #AFE            
        DUS #PARALLEL_DLY   
        LPP #PAR_0          
        LSR #AFE            
        DUS #PARALLEL_DLY   
        LPE                      *  end of parallel binning loop
        DUS #TDIR_DLY
        LPP #SER_1
        LSR #AFE       
        DSC #SERIAL_DLY
        LPP #SER_2
        LSR #AFE       
        DSC #SERIAL_DLY
        LPP #SER_3
        LSR #AFE       
        DSC #SERIAL_DLY
        LPP #SER_4
        LSR #AFE       
        DSC #SERIAL_DLY
        LPP #SER_5
        LSR #AFE       
        DSC #SERIAL_DLY
        LPP #SER_6
        LSR #AFE                 * leave the charge under S1
        RET

#SER_READ                        * read one pixel: shift, process, A/D convert, xmit
        LMR #WRITE32             * Set things up
        LDA #CDSREG              * Comes in with S1
        LPP #CDS00               * #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST 
        LSR #AFE
        LDA #CLKPORT
        LRB #NSBIN               * Load serial binning
       
        LPP #SER_0               *  S1  S2
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_1               *      S2      RG
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_2               *      S2  S3
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_3               * S3 SW
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_4               * S1      S3  SW
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_5               * S1         SW
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 
        
        LPE                      * end of serial binning loop
                                 * Charge is under SW        
        LDA #CDSREG
        LPP #CDS01               * #CDS_INV OR #CDS_CLP OR #CDS_RST
        LSR #AFE
        LPP #CDS02               * #CDS_INV OR #CDS_RST
        LSR #AFE
        LPP #CDS03               * #CDS_INV
        LSR #AFE
        LPP #CDS04               * #CDS_INV OR #CDS_INT
        LSR #AFE

        DSC #CDS_DWELL           * integrate on the reset level

        LPP #CDS05               * #CDS_INV
        LSR #AFE
        LPP #CDS06               * #CDS_INV OR #CDS_NON
        LSR #AFE

        LDA #CLKPORT
        LPP #SER_6               * S1
        LSR #AFE 

        LDA #CDSREG
        LPP #CDS07               * #CDS_NON
        LSR #AFE
        LPP #CDS08               * #CDS_NON OR #CDS_INT
        LSR #AFE

        DSC #CDS_DWELL           * integrate on the video level

        LPP #CDS09               * #CDS_NON
        LSR #AFE
        DSC #ADC_SETTLE          * Wait for the signal to settle on the ADC
        LPP #CDS10               * #CDS_NON OR #CDS_CTC
        LSR #AFE
        DSC #ADC_HOLD
        LPP #CDS11               * #CDS_NON
        LSR #AFE
        RET

#PAR_CLEAR 
        LMR #WRITE32
        LDA #CLKPORT
        LPP #PAR_CLR1   
        LSR #AFE  
        DUS #PARALLEL_CLR_DLY   *  parallel clock overlap time - 74ns
        LPP #PAR_CLR2  
        LSR #AFE          
        DUS #PARALLEL_CLR_DLY 
        LPP #PAR_CLR3  
        LSR #AFE          
        DUS #PARALLEL_CLR_DLY 
        LPP #PAR_CLR4   
        LSR #AFE          
        DUS #PARALLEL_CLR_DLY 
        LPP #PAR_CLR5   
        LSR #AFE          
        DUS #PARALLEL_CLR_DLY 
        LPP #PAR_CLR0   
        LSR #AFE              
        DUS #PARALLEL_CLR_DLY 
        RET
        
#SER_CLEAR
      LMR #WRITE32               * Set the write mode up	
      LDA #CDSREG
      LPP #CDS00
      LSR #AFE                   * CDS00 Put the CDS into safe mode   
      LDA #CLKPORT
      LRB #NCOL                  * Shift NCOL columns to the output
      LPP #SER_CLR0              * S1 S2
      LSR #AFE      
      DSC #SER_CLR_DLY	         * serial clock overlap - 2
      LPP #SER_CLR1              * S2
      LSR #AFE 	
      DSC #SER_CLR_DLY
      LPP #SER_CLR2              * S2 S3 SW
      LSR #AFE 
      DSC #SER_CLR_DLY
      LPP #SER_CLR3              * S3 SW
      LSR #AFE 
      DSC #SER_CLR_DLY
      LPP #SER_CLR4              * S1 S3 SW
      LSR #AFE 
      DSC #SER_CLR_DLY
      LPP #SER_CLR5              * S1 SW
      LSR #AFE
      DSC #SER_CLR_DLY
      LPP #SER_CLR6              * S1
      LSR #AFE         
      DSC #SER_CLR_DLY
      LPE
      LPP #SER_CLR0              * S1 S2     
      LSR #AFE                   * Leave with S1 + S2 + RG ready for the first parallel transfer
      RET

#CDS_TEST
        LMR #WRITE32             * Set things up
        LDA #CDSREG              * Comes in with S1
        LPP #CDS00               * #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST 
        LSR #AFE
        LDA #CLKPORT
      
        LPP #SER_0               *  S1  S2
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_1               *      S2      RG
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_2               *      S2  S3
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_3               * S3 SW
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_4               * S1      S3  SW
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 

        LPP #SER_5               * S1         SW
        LSR #AFE 
        DSC #SERIAL_DLY          * one serial delay time 
                                         * Charge is under SW        
        LDA #CDSREG
        LPP #CDS01               * #CDS_INV OR #CDS_CLP OR #CDS_RST
        LSR #AFE
        LPP #CDS02               * #CDS_INV OR #CDS_RST
        LSR #AFE

        DSC #CDS_DWELL           * integrate on the reset level

        LPP #CDS03               * #CDS_INV
        LSR #AFE
        LRB #TST_DLY               * Load serial binning
        NOP
        LPE
        LPP #CDS04               * #CDS_INV OR #CDS_INT
        LSR #AFE

        DSC #CDS_DWELL           * integrate on the reset level

        LPP #CDS05               * #CDS_INV
        LSR #AFE
        LRB #TST_DLY               * Load serial binning
        NOP
        LPE
        LPP #CDS06               * #CDS_INV OR #CDS_NON
        LSR #AFE

        DSC #CDS_DWELL           * integrate on the reset level

        LPP #CDS07               * #CDS_NON
        LSR #AFE
        LRB #TST_DLY               * Load serial binning
        NOP
        LPE
        LPP #CDS08               * #CDS_NON OR #CDS_INT
        LSR #AFE

        DSC #CDS_DWELL           * integrate on the video level

        LPP #CDS09               * #CDS_NON
        LSR #AFE
        DSC #ADC_SETTLE          * Wait for the signal to settle on the ADC
        LPP #CDS10               * #CDS_NON OR #CDS_CTC
        LSR #AFE
        DSC #ADC_HOLD
        LPP #CDS11               * #CDS_NON
        LSR #AFE
        RET
      
* END OF PROGRAM SEGMENT
