******************************************************
** PROJECT     : TORRENT
** TITLE       : Test sequence for the Dewar heater characterization.
** VERSION     : 1.0
** HISTORY     : VER    DATE     AUTHOR     COMMENTS
**             : 1.0    12/06/11   PCM    Blah Blah Blah
**             : 
** DESCRIPTION : This code is used to validate the design of the Torrent DHE Dewar heater hardware.
**             :
**             : Compile as tasm dewarTest_Sequencer.asm dewarTest_Sequencer.ucd
**             :
**
*
*******************************
** LOOP REGISTER DEFINITIONS ** 
*******************************
DEF #NROW             0       * NUMBER OF ROWS TO READOUT (DET_ROWS / NPBIN)
DEF #NCOL             1       * NUMBER OF COLUMNS TO READOUT (DET_COLS / NSBIN)
DEF #NPBIN            2       * Number to bin Parallels
DEF #NSBIN            3       * Number to bin Serials
DEF #ROWSKIP          4       * Number of ROWS to skip for IDLE
DEF #COLSKIP          5       * Number of COLUMNS to skip for IDLE
DEF #NPRE             7       * Number of serial prescan pixels
DEF #SHUTDLY          8       * CLOCK TICKS TO HOLD INT RESET AND DC RESTORE TRUE
DEF #INITDLY          9       * Clock initialization delay
DEF #ROI_ROWS        10       * ROI - number of rows in ROI
DEF #ROI_COLS        11       * ROI - number of columns in ROI
DEF #ROI_PRE         12       * ROI - number of rows to skip up to the ROI
DEF #ROI_SKIP        13       * ROI - number of cols to skip over to the ROI
DEF #TST_DLY         10
**
**
**
**************************************************************
** JUMP FLAG DEFINITIONS - BIT POSITION IN COMMAND REGISTER ** from stis
**************************************************************
DEF #SYNC             1       * Jump on sync bit high
DEF #ITC              2       * Jump on Integration time expired
DEF #START            3       * Jump on start exposure flag set
DEF #VCTR_1           4       * STRVCTR =  1: do heater ram test
DEF #VCTR_2           5       * STRVCTR =  2:
DEF #VCTR_4           6       * STRVCTR =  4:
DEF #VCTR_8           7       * STRVCTR =  8: 
DEF #VCTR_16          8       * STRVCTR = 16: 
DEF #VCTR_32          9       * STRVCTR = 32:
DEF #VCTR_64         10       * STRVCTR = 64:
DEF #VCTR_128        11       * STRVCTR = 128:
**   User Bits
DEF #CONT_RUN        12       * Continuous run flag
DEF #SBIN_ENB        13       * Serial binning enable bit
DEF #USR_3           14       * Jump on user mode bit value 4
DEF #USR_4           15       * Jump on user mode bit value 8
**
****************************
** EFR REGISTER CONSTANTS ** Used with 16-bit or 32-bit write modes 
****************************
DEF #KILL_EXPFLG    x01080001 * Cancel start exposure flag (bit 3 of Command Register) using the EFR register
DEF #PULSE_PWRSYNC  x01080002 * Pulse the power synchronization strobe for power supply synchronization
DEF #STRT_INTCNTR   x01080004 * Start the integration time counter using the EFR register
DEF #STOP_INTCNTR   x01080008 * Stop the integration time counter using the EFR register
DEF #OPEN_SHUTTER   x01080010 * Command the shutter to open
DEF #CLOSE_SHUTTER  x01080020 * Command the shutter to close
DEF #PREFLASH_ON    x01080040 * Turn the preflash signal on
DEF #PREFLASH_OFF   x01080080 * Turn off the preflash signal
DEF #PULSE_FRAME    x01080100 * Pulse the Frame Start flag to indicate a new frame
DEF #PULSE_LINE     x01080200 * Pulse the Line s Start Flag to indicate a new line beginning
DEF #READOUT_BUSY   x01080400 * Set the readout in progress flag to the DHE - Allows shutting down certain services while reading out.
DEF #READOUT_IDLE   x01080800 * Set the readout in progress flag false
DEF #SPARE_EFR_1    x01081000 * Spare toggle bit - not used in sequencer
DEF #SPARE_EFR_2    x01082000 * Spare toggle bit - not used in sequencer
DEF #SET_SYNC_LOW   x01084000 * If enabled as master DHE, set sync bit low after sync delay
DEF #SET_SYNC_HIGH  x01088000 * If enabled as master DHE, set sync signal high immediately
*
**   COMBINATION EFR COMMAND WORDS             
DEF #OPEN_AND_START      #STRT_INTCNTR OR #OPEN_SHUTTER     * START INTEGRATION TIMER AND OPEN SHUTTER
DEF #CLOSE_AND_STOP      #STOP_INTCNTR OR #CLOSE_SHUTTER    * CLOSE SHUTTER AND STOP INTEGRATION TIMER
DEF #CLOSE_STOP_BUSY     #CLOSE_AND_STOP OR #READOUT_BUSY   * CLOSE SHUTTER, STOP INTEGRATION TIMER, AND SET 'READOUT IN PROGRESS' STATUS
DEF #CLOSE_AND_START     #CLOSE_SHUTTER OR #STRT_INTCNTR    * CLOSE SHUTTER AND START INTEGRATION COUNTER - FOR DARK EXPOSURES
**
*****************************
** MODE REGISTER CONSTANTS **
*****************************
DEF #RESET            0       * Mode register constant for board reset operation - Not used in sequencer
DEF #READ             1       * Mode register constant for read operation - Not used in sequencer
DEF #WRITE16          2       * Mode register constant for 16 bit write operation - Module address contained in upper 16-bits of pattern data
DEF #WRITE32          3       * Mode register constant for 32 bit write operation - use LDA to adress module
**
*******************************
** LSR BOARD SELECT ASSIGNMENTS ** combined
*******************************
DEF #DSLCT            0
DEF #LCB              1
DEF #PSM              2
DEF #CFG              4
DEF #PIX              8
DEF #AFE             16
**
*******************************
** PSM MODULE REGISTER ADDRESSING
DEF #HTRPWMPORT   x010A
**
*******************************
** CFG MODULE REGISTER ADDRESSING
DEF #EFR_REG      x0108       * WRITE ALL EFR PARAMETERS TO THIS REGISTER WHEN USING 32-BIT MODE
DEF #TSTCOND_REG  x0109       * WRITE A 23-BIT ADDRESS HERE TO SEE the RESULT OF A READ OF THAT ADDRESS HERE
**
*******************************
** TEST CONDITION READ ADDRESSES
DEF #RD_STATUS x0001fffd            * read the system ststus word
**
*******************************
** AFE Sub addresses
DEF #CLKPORT      x3002       * AFE MODULE CLOCK PORT POINTER
DEF #CDSREG       x2002       * AFE MODULE CDS PORT POINTER
DEF #SEQCMD       x2003       * AFE CDS SEQUENCER TRIGGER POINTER
**
*******************************
** CDS state values - written as data for 16-bit mode writes to the cds register
*******************************
DEF #CDS_CTC  x20020001       * CTC strobe
DEF #CDS_INV  x20020002       * INVERT mode
DEF #CDS_NON  x20020004       * NON_INVERT mode
DEF #CDS_INT  x20020008       * Integrate
DEF #CDS_CLP  x20020010       * DC Restore Clamp
DEF #CDS_RST  x20020020       * RESET Integratoe

********************************
** TEST VECTORS - These are the equivelent values for 0% -> 100% power steps in incre,memts of 10% 
DEF #HTRPWRVALUE_00    0
DEF #HTRPWRVALUE_10   13
DEF #HTRPWRVALUE_20   26
DEF #HTRPWRVALUE_30   39
DEF #HTRPWRVALUE_40   51
DEF #HTRPWRVALUE_50   64
DEF #HTRPWRVALUE_60   77
DEF #HTRPWRVALUE_70   89
DEF #HTRPWRVALUE_80  102
DEF #HTRPWRVALUE_90  114
DEF #HTRPWRVALUE_100 127
**
********************************
** PATTERN MEMORY DEFINITIONS ** combined
********************************
** PATTERN MEMORY SEGMENTATION MAP
DEF #PATSEG_00   x0000       * Initialization stuff
DEF #PATSEG_02   x0020       * CCD READ INITIALIZATION
DEF #PATSEG_03   x0030       * CCD PARALLEL SHIFT VECTORS
DEF #PATSEG_04   x0040       * CCD SERIAL SHIFT VECTORS
DEF #PATSEG_08   x0080       * CDS vectors
DEF #PATSEG_0A   x00A0       * Test condition read addresses
DEF #PATSEG_0C   x00C0       * Place to put all ROI shifting

ORG PAT #PATSEG_00                    ********** GENERAL CONSTANTS KEPT IN PAT MEM ********* 
PAT #KILL_EXPFLG     #KILL_EXPFLG     ** We can use the same label name here because they are in different contexts
PAT #PULSE_PWRSYNC   #PULSE_PWRSYNC   ** the definition earlier points to a data value. the label here points to
PAT #STRT_INTCNTR    #STRT_INTCNTR    ** a pattern memory address
PAT #STOP_INTCNTR    #STOP_INTCNTR    ********************
PAT #OPEN_SHUTTER    #OPEN_SHUTTER    
PAT #CLOSE_SHUTTER   #CLOSE_SHUTTER   
PAT #PREFLASH_ON     #PREFLASH_ON     
PAT #PREFLASH_OFF    #PREFLASH_OFF    
PAT #PULSE_FRAME     #PULSE_FRAME     
PAT #PULSE_LINE      #PULSE_LINE      
PAT #READOUT_BUSY    #READOUT_BUSY    
PAT #READOUT_IDLE    #READOUT_IDLE    
PAT #SET_SYNC_LOW    #SET_SYNC_LOW    
PAT #SET_SYNC_HIGH   #SET_SYNC_HIGH  
PAT #OPEN_AND_START  #OPEN_AND_START
PAT #CLOSE_AND_STOP  #CLOSE_AND_STOP
PAT #CLOSE_STOP_BUSY #CLOSE_STOP_BUSY
PAT #CLOSE_AND_START #CLOSE_AND_START
                  
ORG PAT #PATSEG_02      ** Instantuate the Dewar heater power test vectors ********* 
PAT #HTRPWRVALUE_00    #HTRPWRVALUE_00 
PAT #HTRPWRVALUE_10    #HTRPWRVALUE_10 
PAT #HTRPWRVALUE_20    #HTRPWRVALUE_20 
PAT #HTRPWRVALUE_30    #HTRPWRVALUE_30 
PAT #HTRPWRVALUE_40    #HTRPWRVALUE_40 
PAT #HTRPWRVALUE_50    #HTRPWRVALUE_50 
PAT #HTRPWRVALUE_60    #HTRPWRVALUE_60 
PAT #HTRPWRVALUE_70    #HTRPWRVALUE_70 
PAT #HTRPWRVALUE_80    #HTRPWRVALUE_80 
PAT #HTRPWRVALUE_90    #HTRPWRVALUE_90 
PAT #HTRPWRVALUE_100   #HTRPWRVALUE_100

*
**
*******************************
** Program code  starts here **
*******************************
** EXECUTIVE ROUTINE - WAIT HERE FOR WORK
*
#INIT   NOP                     * Give the sequencer time to fill teh instruction pipeline
#EXEC   JCB #GOBABY #START      * IF START EXPOSURE FLAG SET GO AND IDENTIFY THE COMMAND 
        JMP #EXEC               * AND KEEP LOOKING FOR WORK
  
#GOBABY LMR #WRITE32
        LDA #EFR_REG            * POINT TO THE FLAG REGISTER
        LPP #KILL_EXPFLG        * POINT TO THE APPROPRIATE PATTERN DATA TO CANCEL THE START EXP FLAG - PATTERN = 0x00000001
        LSR #CFG                * WRITE FLAG REGISTER IN CONFIG FIRMWARE MODULE
        JCB #JVCTR1   #VCTR_1   * START VECTOR = 1
        JCB #JVCTR2   #VCTR_2   * START VECTOR = 2
        JCB #JVCTR4   #VCTR_4
        JCB #JVCTR8   #VCTR_8
        JCB #JVCTR16  #VCTR_16
        JCB #JVCTR32  #VCTR_32
        JCB #JVCTR64  #VCTR_64
        JCB #JVCTR128 #VCTR_128         
        JMP #EXEC               * NO FLAGS SET, BACK TO THE EXEC LOOP
*
**
***********************         
** FUNCTION ROUTINES **
***********************
**
*
* START VECTOR = 1 - Run the htrServoPwmValue ramp test for the heater power linearity test
#JVCTR1 CAL #PWRTST
        JCB #JVCTR1 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 2 - CLEAR THE DETector, dark integration (no shutter) and normal readout
#JVCTR2 
*        JCB #JVCTR2 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 4 - SIMPLE READOUT
#JVCTR4
*        JCB #JVCTR4 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 8 - 
#JVCTR8
*        JCB #JVCTR8 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC
          
* START VECTOR = 16 - 
#JVCTR16
*        JCB #JVCTR16 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 32 - 
#JVCTR32
*        JCB #JVCTR32 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 64 - 
#JVCTR64
*        JCB #JVCTR64 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC

* START VECTOR = 128 - 
#JVCTR128
*        JCB #JVCTR128 #CONT_RUN       * do it all again if told to do so
        JMP #EXEC
*
**        
*****************   
** SUBROUTINES **
*****************  
**
#PWRTST LMR #WRITE32  
        LPP #HTRPWRVALUE_00
        LDA #HTRPWMPORT
        LRB #NROW               * loop for n rows
        LSR #PSM
        CAL #DLY_ONESEC
        CAL #DLY_ONESEC
        CAL #DLY_ONESEC
        IPP
        LPE
        LPP #HTRPWRVALUE_00
        LDA #HTRPWMPORT
        RET
       
#DLY_ONESEC        
        DMS 200
        DMS 200
        DMS 200
        DMS 200
        DMS 200
        RET

* END OF PROGRAM SEGMENT
